#pragma rtGlobals=1		// Use modern global access method.

// Demonstration of Managing Multiple Versions of a Procedure Package

// To try this demo:
//	Read the rest of the comments below to understand what the demo is all about.
//	Put the "Demo" folder in "Igor Pro 6 User Files"
//	Make a shortcut from "Demo:Demo Loader.ipf" and put it in "Igor Pro 6 User Files:Igor Procedures"
//	Restart Igor

// Background
// You should avoid supporting old versions of your package if it is possible.
// Life is difficult enough without it. When you create a new version of your
// procedures, freeze the old version and let existing users continue it
// until it is convenient to update. Focus your efforts on improving the new
// version only.

// If you must support multiple versions, here is an example of how you might do it.
// In this example we assume that the package name is "Demo". To use this in
// your own project, clone the Demo folder, rename it with the name of your package,
// and change "Demo" to the name of your package in all of the files including this one.

// Imagine an organization like this:
// 	Igor Pro 6 User Files
// 		Igor Procedures
// 			<Shortcut to Demo Loader.ipf>
// 		Demo								// Folder containing everything relating to the package
// 			Demo Loader.ipf
// 			V1								// Version 1 files
// 				Demo V1 Template.pxt
// 				Procedures
//					DemoProc.ipf
//			V2								// Version 2 files
//				Demo V2 Template.pxt
//				Procedures
//					DemoProc.ipf
//			V3								// Version 3 files
//				Demo V3 Template.pxt
//				Procedures
//					DemoProc.ipf
//
// You install the Demo folder as shown and install a shortcut from "Demo Loader.ipf"
// into "Igor Procedures". This adds a "Demo" menu to the menu bar. This menu would,
// among other things, contain items for loading a particular version of the package.
//
// The "Demo V1 Template.pxt" in V1 would have, in the built-in procedure window:
//	#include ":Demo:Procedures:V1:DemoProc"
//
// The "Version 2 Template.pxt" in V2 would have, in the built-in procedure window:
//	#include ":Demo:Procedures:V2:DemoProc"
//
// To switch to a new version the user would simply change V1 to V2 in the
// #include statement in the built-in procedure window or choose Demo->Load Version 2,
// for example, from the Demo menu.
//
// To update the procedures the user would simply replace the entire "Demo" folder with a new version.
// He would have to quit Igor or make sure no Demo files are open to do this.
//
// If the package was hosted in a Subversion repository, for example, this could be done by simply updating
// the users' working copy.
//
// To maintain backward compatibility, you would never delete a version and never change an old version,
// except for minor changes that you think have very low probability of causing bugs.
//
// Each version should store its own copy of package data in, for example "root:Packages:Demo:V1"
// and "root:Packages:Demo:V2". This gives you the freedom to change your data structures.
// You could possible allow one version, e.g., V2, to import the package data of an older version, e.g., V1.
//
// Note that the user should not store his own files in the "Demo" folder.
// If possible, I would write-protect that folder to enforce this.
//
// The "Demo" folder could be a shortcut that points to a network folder.

Menu "Demo"
	"Load Version 1", /Q,  LoadDemoPackage(1)
	"Load Version 2", /Q, LoadDemoPackage(2)
	"Load Version 3", /Q, LoadDemoPackage(3)
	"Unload Demo Procedures", /Q, UnloadDemoProcedures()
End

Function UnloadDemoProcedures()
	Execute/P "DELETEINCLUDE \":Demo:V1:Procedures:DemoProc\""
	Execute/P "DELETEINCLUDE \":Demo:V2:Procedures:DemoProc\""
	Execute/P "DELETEINCLUDE \":Demo:V3:Procedures:DemoProc\""
	Execute/P "COMPILEPROCEDURES "		// Note trailing space is required
End

Function LoadDemoPackage(version)
	Variable version		// e.g., 1
	
	String path = ""

	switch(version)
		case 1:
			path = ":Demo:V1:Procedures:DemoProc"
			break
		case 2:
			path = ":Demo:V2:Procedures:DemoProc"
			break
		case 3:
			path = ":Demo:V3:Procedures:DemoProc"
			break
		default:
			Abort "Unknown Demo version"
			break
	endswitch

	UnloadDemoProcedures()	
	
	Execute/P "INSERTINCLUDE " + "\"" + path + "\""
	Execute/P "COMPILEPROCEDURES "		// Note trailing space is required
	Execute/P "AboutDemoPackage()"
End
